package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;


/** ------------------------------------------------------------------------------------------------------------------
 * start of ARFFselectFeatures class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class ARFFselectFeatures {

	
	
	
	/*  global parameters  */
	private static boolean verbose = true;
	private static String inputPath = "";
	private static String outputPath = "";
	/*  end of global parameters  */
	
	
	
	
	/*  start of loadSelectedFeatures method  */
	/**
	 * @param filename_selected_features the name of the file containing the list of features to select
	 */
	private static ArrayList<String> loadSelectedFeatures(String filename_selected_features){
		if(filename_selected_features == null){
			System.err.println("Error: the filename for the selected features list is null.\nSystem.exit(0)");
			System.exit(0);
		}
		
		ArrayList<String> selected_features_list = new ArrayList<String>();
		

		int lineCounter = -1;
		String line = null;
		try{
			BufferedReader br = new BufferedReader(new FileReader(inputPath + filename_selected_features));			
			while((line = br.readLine()) != null){
				if(lineCounter >= 0){		//  there is one header line to skip
					//  make sure that the feature is not already in the list
					if(selected_features_list.contains(line)){
						System.err.println("Error: there is a duplicate entry for feature '" + line + "' in the input selected features list file.\nSystem.exit(0)");
						System.exit(0);
					}
					selected_features_list.add(line);
				}
				else{
					//  display and output the header line
//					System.out.println(line);
				}
				lineCounter++;
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
			System.err.println("Error: cannot find file with name \"" + inputPath + filename_selected_features + "\"\nSystem.exit(0)");
			System.exit(0);
		}
		if(verbose){
			System.out.println("\n\tOpened and read file: " + inputPath + filename_selected_features);
		}
		selected_features_list.trimToSize();		
		
		return selected_features_list;
	}/*  end of loadSelectedFeatures method  */
	
	
	
	
	/*  start of displaySelectedFeaturesList method  */
	/**
	 * @param selected_features_list the list of selected features to display
	 */
	private static void displaySelectedFeaturesList(ArrayList<String> selected_features_list){
		if(selected_features_list == null){
			System.err.println("Error: the selected features list is null and cannot be displayed.\nSystem.exit(0)");
			System.exit(0);
		}
		
		System.out.println("\n=====  Selected Features List  =====");
		for(String current_feature_name: selected_features_list){
			System.out.println("\t" + current_feature_name);
		}
		System.out.println("====================================\n");
	}/*  end of displaySelectedFeaturesList method  */
	
	
	
	
	/*  start of loadARFFfilenames method  */
	/**
	 * @param filename_ARFF_filelist the name of the file containing the list of ARFF filenames
	 */
	private static ArrayList<String> loadARFFfilenames(String filename_ARFF_filelist){
		if(filename_ARFF_filelist == null){
			System.err.println("Error: the filename for the ARFF file list is null.\nSystem.exit(0)");
			System.exit(0);
		}
		
		ArrayList<String> filenames_ARFF_files = new ArrayList<String>();
		

		int lineCounter = -1;
		String line = null;
		try{
			BufferedReader br = new BufferedReader(new FileReader(inputPath + filename_ARFF_filelist));			
			while((line = br.readLine()) != null){
				if(lineCounter >= 0){		//  there is one header line to skip
					filenames_ARFF_files.add(line);
				}
				else{
					//  display and output the header line
//					System.out.println(line);
				}
				lineCounter++;
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
			System.err.println("Error: cannot find file with name \"" + inputPath + filename_ARFF_filelist + "\"\nSystem.exit(0)");
			System.exit(0);
		}
		if(verbose){
			System.out.println("\n\tOpened and read file: " + inputPath + filename_ARFF_filelist);
		}
		filenames_ARFF_files.trimToSize();		
		
		return filenames_ARFF_files;
	}/*  end of loadARFFfilenames method  */

	
	
	
	/*  start of displayARFFfilenames method  */
	/**
	 * @param filenames_ARFF_files the list of ARFF filenames to display
	 */
	private static void displayARFFfilenames(ArrayList<String> filenames_ARFF_files){
		if(filenames_ARFF_files == null){
			System.err.println("Error: the ARFF filenames list is null and cannot be displayed.\nSystem.exit(0)");
			System.exit(0);
		}
		
		System.out.println("\n==========  ARFF Filenames  ==========");
		for(String current_ARFF_filename: filenames_ARFF_files){
			System.out.println(current_ARFF_filename);
		}
		System.out.println("======================================\n");
	}/*  end of displayARFFfilenames method  */
	
	
	
	
	/*  start of filterARFFforSelectedFeaturesOnly method  */
	/**
	 * @param filenames_ARFF_files the list of ARFF filenames to filter
	 * @param selected_features_list the list of selected features to use to filter the ARFF files
	 */
	private static void filterARFFforSelectedFeaturesOnly(ArrayList<String> filenames_ARFF_files, ArrayList<String> selected_features_list){
		if((filenames_ARFF_files == null) || (selected_features_list == null)){
			System.err.println("Error: the ARFF filenames list and selected features list must both be non-null to filter the ARFF files for the selected features.\nSystem.exit(0)");
			System.exit(0);
		}
		
		
		//  go through each ARFF file one-at-a-time
		for(String filename_currentARFF: filenames_ARFF_files){
			
			//  create the output filename
			String filename_output_currentARFF = filename_currentARFF.substring(0, filename_currentARFF.lastIndexOf(".")) + "_selected_features.arff";
			
			ArrayList<Integer> indicesOfSelectedFeatures = new ArrayList<Integer>();
			
			try{
				BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath + filename_output_currentARFF));
				BufferedReader br = new BufferedReader(new FileReader(inputPath + filename_currentARFF));			
				
				
				boolean directCopyMode = true;
				boolean featureIndexFindingMode = false;
				boolean selectedDataOutputMode = false;
				
				int lineIndexForCurrentFeature = 0;
				
				String line = null;
				while((line = br.readLine()) != null){
					if(directCopyMode){
						if(line.startsWith("@ATTRIBUTE")){
							featureIndexFindingMode = true;
							lineIndexForCurrentFeature = 0;
							directCopyMode = false;
						}
						else{
							bw.write(line);
							bw.newLine();
							bw.flush();
						}
					}
					if(featureIndexFindingMode){
						//  print and skip any blank lines
						if(line.equals("")){
							bw.write(line);
							bw.newLine();
							bw.flush();
						}
						else if(line.startsWith("@DATA")){
							directCopyMode = false;
							featureIndexFindingMode = false;
							selectedDataOutputMode = true;
							
							bw.write(line);
							bw.newLine();
							bw.flush();
						}
						else{
							//  test if the current feature is part of the selected list
							String[] values = line.split(" ");
							if(values.length != 3){
								System.out.println("'" + line + "'" + "\tlength = " + values.length);
								System.err.println("Error: each attribute line of the ARFF file does not contain exactly three (3) entries.\nSystem.exit(0)");
								System.exit(0);
							}
							String current_feature_name = values[1];

							if(selected_features_list.contains(current_feature_name)){
								indicesOfSelectedFeatures.add(lineIndexForCurrentFeature);

								//  write the current selected feature to the output file
								bw.write(line);
								bw.newLine();
								bw.flush();
							}

							lineIndexForCurrentFeature++;
						}
					}
					else if(selectedDataOutputMode){
						indicesOfSelectedFeatures.trimToSize();
						
						//  split the line into an array containing each of the feature values
						String[] values = line.split(",");
						
						//  only output the values at the indices corresponding to the selected features
						for(int i=0; i<indicesOfSelectedFeatures.size(); i++){
							int indexOfSelectedFeatureToOutput = indicesOfSelectedFeatures.get(i);
							
							bw.write(values[indexOfSelectedFeatureToOutput]);
							
							if(i != indicesOfSelectedFeatures.size()-1){
								bw.write(",");
							}
							bw.flush();
						}
						bw.newLine();
						bw.flush();
					}
				}
				bw.flush();
				bw.close();
				br.close();
			}catch (IOException e) {
				e.printStackTrace();
				System.err.println("Error: cannot find file with name \"" + inputPath + filename_currentARFF + "\"\nSystem.exit(0)");
				System.exit(0);
			}
			
			if(verbose){
				System.out.println("\n\tOpened and read file: " + inputPath + filename_currentARFF);
				System.out.println("\tOutput file: " + outputPath + filename_output_currentARFF);
			}
		}
	}/*  end of filterARFFforSelectedFeaturesOnly method  */
	
	
	
	
	/*  start of main method  */
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		
				
		/******************************************************
		 * 													  *
		 * 				SET THE PARAMETERS HERE				  *
		 * 													  *
		 ******************************************************/
		
		
		inputPath = "ARFFselectFeatures/Input/";
		outputPath = "ARFFselectFeatures/Output/";

		
		String filename_ARFF_filelist = "file_list_[training_and_tests].txt";
		
		String filename_selected_features_list = "selected_features_list_[p<0.01].txt";
		
		

		/******************************************************
		 * 													  *
		 * 				END OF SETTING THE PARAMETERS		  *
		 * 													  * 
		 ******************************************************/
		
		
		
		
		//  load the list of features to select
		ArrayList<String> selected_features_list = loadSelectedFeatures(filename_selected_features_list);
		if(verbose){
			displaySelectedFeaturesList(selected_features_list);
		}
		
		
		//  load the list of ARFF filenames
		ArrayList<String> filenames_ARFF_files = loadARFFfilenames(filename_ARFF_filelist);
		if(verbose){
			displayARFFfilenames(filenames_ARFF_files);
		}
		
		
		//  create new files containing only the selected features
		filterARFFforSelectedFeaturesOnly(filenames_ARFF_files, selected_features_list);
		
		
		if(verbose){
			System.out.println("Done!");
		}

	}/*  end of main method  */

}/*  end of ARFFselectFeatures class  */
