package analysis;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.util.ArrayList;


/** ------------------------------------------------------------------------------------------------------------------
 * start of BurnInAnalyzerMain class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class BurnInAnalyzerMain {

	/*  start of main method  */
	public static void main(String[] args) {
		
		
		
		
		boolean verbose = true;
		boolean usingCommandLine = false;
		
		int numberOfStructures = 0;
		int windowSize = 10;
		int numberOfRunsPerExperimentName = 2;
		ArrayList<String> experimentNames = new ArrayList<String>();
	
		String inputPath = "";
		String outputPath = "";
		
		
		//  load the parameters from the command line
		if(args.length >= 4){
			//  first parameter is the number of structures per run
			numberOfStructures = Integer.parseInt(args[0]);
			
			//  second parameter is the window size
			windowSize = Integer.parseInt(args[1]);
			
			//  third parameter is the number of runs per experiment name
			numberOfRunsPerExperimentName = Integer.parseInt(args[2]);
			
			//  all subsequent parameters are the filenames of the experiment files
			for(int i=3; i<args.length; i++){
				experimentNames.add(args[i]);
			}
			experimentNames.trimToSize();
			
			usingCommandLine = true;
			verbose = false;
		}
		else{
			System.err.println("Error: there need to be at least four (4) command line parameters.\n" +
					"\t\tnumber_of_structures_per_run\n" +
					"\t\twindow_size\n" +
					"\t\tnumber_of_runs_per_experiment_name\n" +
					"\t\texperiment_1_name\t\t\t(just the name substring in 'structure_####_experimentName[i].pdb')\n" +
					"\t\texperiment_2_name\n" +
					"\t\texperiment_..._name\n" +
					"System.exit(0)");
//			System.exit(0);
		}
		
		
		
		/******************************************************
		 * 													  *
		 * 				SET THE PARAMETERS HERE				  *
		 * 													  *
		 ******************************************************/
		
		
		if(!usingCommandLine){
		

			inputPath = "BurnInAnalyzer/Input/";
			outputPath = "BurnInAnalyzer/Output/";
			
			
			//  specify the number of structures from each experiment 
			numberOfStructures = 72;
			
			
			//  specify the size of the sliding window
			windowSize = 25;
			
			
			//  specify the number of runs per experiment name
			numberOfRunsPerExperimentName = 4;
			
			
			//  specify the names of the experiments
			//  this is the common suffix to all structure filenames from each of the runs (i.e. filenames = structure_####_experimentName.pdb)
			
			experimentNames.add("BM-AF9-2");
			
			
			//  STRUCTURE PREDICT DISEASE
//			experimentNames.add("BM-A-1");
//			experimentNames.add("BM-A-2");
//			experimentNames.add("BM-A-3");
			
//			experimentNames.add("BM-H-2");
//			experimentNames.add("BM-H-3");
//			experimentNames.add("BM-H-4");
			
//			experimentNames.add("BM-AF9-1");
//			experimentNames.add("BM-AF9-2");
			
//			experimentNames.add("Nalm-6-1");
//			experimentNames.add("Nalm-6-2");
			
//			experimentNames.add("NB-4-1");
			
			
//			verbose = false;
			
			
		}
		
		
		/******************************************************
		 * 													  *
		 * 				END OF SETTING THE PARAMETERS		  *
		 * 													  * 
		 ******************************************************/
		
		
		
		
		experimentNames.trimToSize();
		
		
		if(verbose){
			System.out.println("There are " + experimentNames.size() + " experiment names, each with " + numberOfRunsPerExperimentName + " runs.");
		}
		
		
		for(String currentExperimentName: experimentNames){
			
			for(int i=1; i<numberOfRunsPerExperimentName; i++){
				for(int j=i+1; j<=numberOfRunsPerExperimentName; j++){
					
					String experimentNameOne = currentExperimentName + "[" + i + "]";
					String experimentNameTwo = currentExperimentName + "[" + j + "]";
					
					
					//  add all of the parameters to the arguments array
					args = new String[4];
					args[0] = experimentNameOne;
					args[1] = experimentNameTwo;
					args[2] = String.valueOf(numberOfStructures);
					args[3] = String.valueOf(windowSize);
				
				
					//  BurnInAnalyzer program object
					BurnInAnalyzer burnInAnalyzer = new BurnInAnalyzer(inputPath, outputPath, verbose);
				
				
					//  run the BurnInAnalyzer program
					burnInAnalyzer.run(args);
				
				}
			}
		}
		
		
		if(verbose){
			System.out.println("Done!");
		}
	}/*  end of main method  */

}/*  end of BurnInAnalyzerMain class  */
