package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;


/** ------------------------------------------------------------------------------------------------------------------
 * start of FiveC class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class FiveC {
	
	
	
	
	/*  global parameters  */
	private String inputPath;
	private String outputPath;	
	private String filename;
	private HashMap<String, Double> probeLength15;
	private HashMap<String, Double> probeLength18;
	private HashMap<String, Double> probeLength19;
	private HashMap<String, Double> probeLength20;
	private HashMap<String, Double> probeLength21;
	private HashMap<String, Double> probeLength22;
	private HashMap<String, Double> probeLength23;
	private HashMap<String, Double> probeLength24;
	private ArrayList<String> allKeys;
	/*  end of global parameters  */
	
	
	

	/*  start of constructor method  */
	/**
	 * @param filename the filename for the file containing the 5C signal intensity values
	 */
	public FiveC(String filename){
		this(filename, "", "");
	}/*  end of constructor method  */
	
	
	
	
	/*  start of constructor method  */
	/**
	 * @param filename the filename for the file containing the 5C signal intensity values
	 * @param inputPath the file system path to access the subfolder used for the input files
	 * @param outputPath the file system path to access the subfolder used for the output files
	 */
	public FiveC(String filename, String inputPath, String outputPath){
		if((filename == null) || (inputPath == null) || (outputPath == null)){
			System.err.println("Error: the filename and paths must be non-null.\nSystem.exit(0)");
			System.exit(0);
		}
		
		this.filename = filename;
		this.inputPath = inputPath;
		this.outputPath = outputPath;
		
		
		this.loadValuesFromFile();
		
		
	}/*  end of constructor method  */
	
	
	
	
	/*  start of loadValuesFromFile method  */
	/**
	 * @return void
	 */
	private void loadValuesFromFile(){
		if(this.filename == null){
			System.err.println("Error: the filename is null and the values cannot be loaded from file.\nSystem.exit(0)");
			System.exit(0);
		}
		
		
		this.allKeys = new ArrayList<String>();
		
		
		this.probeLength15 = new HashMap<String, Double>();
		this.probeLength18 = new HashMap<String, Double>();
		this.probeLength19 = new HashMap<String, Double>();
		this.probeLength20 = new HashMap<String, Double>();
		this.probeLength21 = new HashMap<String, Double>();
		this.probeLength22 = new HashMap<String, Double>();
		this.probeLength23 = new HashMap<String, Double>();
		this.probeLength24 = new HashMap<String, Double>();
		

		String line = null;
		try{
			BufferedReader br = new BufferedReader(new FileReader(this.inputPath + this.filename));
			while((line = br.readLine()) != null){
				String[] values = line.split("\t");
				if(values.length != 10){
					System.err.println("Error: there are not exactly ten (10) entries in each row of the input signal file.\nSystem.exit(0)");
					System.exit(0);
				}
				
				
				//  create the key from the probe indices
				//  always have the indices in order of 'smaller-larger'
				int firstIndex = Integer.parseInt(values[0]);
				int secondIndex = Integer.parseInt(values[1]);
				if(secondIndex < firstIndex){
					int tempIndex = secondIndex;
					secondIndex = firstIndex;
					firstIndex = tempIndex;
				}
				if(firstIndex == secondIndex){
					System.err.println("Error: the first and second indices are the same.\nSystem.exit(0)");
					System.exit(0);
				}
				
				String key = firstIndex + "-" + secondIndex;
				String reverseKey = secondIndex+ "-" + firstIndex ;
				
				if(!this.allKeys.contains(key)){
					this.allKeys.add(key);
				}
				
				double signal15 = Double.parseDouble(values[2]);
				double signal18 = Double.parseDouble(values[3]);
				double signal19 = Double.parseDouble(values[4]);
				double signal20 = Double.parseDouble(values[5]);
				double signal21 = Double.parseDouble(values[6]);
				double signal22 = Double.parseDouble(values[7]);
				double signal23 = Double.parseDouble(values[8]);
				double signal24 = Double.parseDouble(values[9]);
				
				if(signal15 > 0){
					if(this.probeLength15.containsKey(key) || this.probeLength15.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength15.put(key, signal15);
				}
				if(signal18 > 0){
					if(this.probeLength18.containsKey(key) || this.probeLength18.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength18.put(key, signal18);
				}
				if(signal19 > 0){
					if(this.probeLength19.containsKey(key) || this.probeLength19.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength19.put(key, signal19);
				}
				if(signal20 > 0){
					if(this.probeLength20.containsKey(key) || this.probeLength20.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength20.put(key, signal20);
				}
				if(signal21 > 0){
					if(this.probeLength21.containsKey(key) || this.probeLength21.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength21.put(key, signal21);
				}
				if(signal22 > 0){
					if(this.probeLength22.containsKey(key) || this.probeLength22.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength22.put(key, signal22);
				}
				if(signal23 > 0){
					if(this.probeLength23.containsKey(key) || this.probeLength23.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength23.put(key, signal23);
				}
				if(signal24 > 0){
					if(this.probeLength24.containsKey(key) || this.probeLength24.containsKey(reverseKey)){
						System.err.println("Error: the key (" + key + ") is already contained in the HashMap.\nSystem.exit(0)");
						System.exit(0);
					}
					this.probeLength24.put(key, signal24);
				}
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
		}
//		System.out.println("Loaded input file: " + this.inputPath + this.filename);
	}/*  end of loadValuesFromFile method  */
	
	
	
	
	/*  start of normalize method  */
	/**
	 * @param normalizationFactors an array containing the normalization factors for each probe length in order (15bp, 18bp, 19bp, 20bp, 21bp, 22bp, 23bp, 24bp) (the values are multiplied by the normalization factor)
	 */
	public void normalize(double[] normalizationFactors){
		if(normalizationFactors == null){
			System.err.println("Error: the normalizationFactors are null and cannot be used to normalize the data.\nSystem.exit(0)");
			System.exit(0);
		}
		
		double normFactor15 = normalizationFactors[0];
		double normFactor18 = normalizationFactors[1];
		double normFactor19 = normalizationFactors[2];
		double normFactor20 = normalizationFactors[3];
		double normFactor21 = normalizationFactors[4];
		double normFactor22 = normalizationFactors[5];
		double normFactor23 = normalizationFactors[6];
		double normFactor24 = normalizationFactors[7];
		
		for(String currentKey: allKeys){
			
			if(this.probeLength15.containsKey(currentKey)){
				double value15 = this.probeLength15.get(currentKey);
				value15 *= normFactor15;
				this.probeLength15.put(currentKey, value15);
			}
			
			if(this.probeLength18.containsKey(currentKey)){
				double value18 = this.probeLength18.get(currentKey);
				value18 *= normFactor18;
				this.probeLength18.put(currentKey, value18);
			}

			if(this.probeLength19.containsKey(currentKey)){
				double value19 = this.probeLength19.get(currentKey);
				value19 *= normFactor19;
				this.probeLength19.put(currentKey, value19);
			}

			if(this.probeLength20.containsKey(currentKey)){
				double value20 = this.probeLength20.get(currentKey);
				value20 *= normFactor20;
				this.probeLength20.put(currentKey, value20);
			}

			if(this.probeLength21.containsKey(currentKey)){
				double value21 = this.probeLength21.get(currentKey);
				value21 *= normFactor21;
				this.probeLength21.put(currentKey, value21);
			}

			if(this.probeLength22.containsKey(currentKey)){
				double value22 = this.probeLength22.get(currentKey);
				value22 *= normFactor22;
				this.probeLength22.put(currentKey, value22);
			}

			if(this.probeLength23.containsKey(currentKey)){
				double value23 = this.probeLength23.get(currentKey);
				value23 *= normFactor23;
				this.probeLength23.put(currentKey, value23);
			}

			if(this.probeLength24.containsKey(currentKey)){
				double value24 = this.probeLength24.get(currentKey);
				value24 *= normFactor24;
				this.probeLength24.put(currentKey, value24);
			}
		}
	}/*  end of normalize method  */
	
	
	
	
	/*  start of getFilename method  */
	/**
	 * @return the filename of the 5C experiment
	 */
	public String getFilename(){
		return this.filename;
	}/*  end of getFilename method  */
	
	
	
	
	/*  start of getExperimentName method  */
	/**
	 * @return the name of the 5C experiment
	 */
	public String getExperimentName(){
		if(this.filename == null){
			System.err.println("Error: the filename is null and the experiment name cannot be determined.\nSystem.exit(0)");
			System.exit(0);
		}
		
		return this.filename.substring(0, this.filename.lastIndexOf("."));
	}/*  end of getExperimentName method  */
	
	
	
	
	/*  start of getInputPath method  */
	/**
	 * @return the InputPath set for the current FiveC object
	 */
	public String getInputPath(){
		return this.inputPath;
	}/*  end of getInputPath method  */
	
	
	
	
	/*  start of getOutputPath method  */
	/**
	 * @return the OutputPath set for the current FiveC object
	 */
	public String getOutputPath(){
		return this.outputPath;
	}/*  end of getOutputPath method  */
	
	
	

	/*  start of getAllKeys method  */
	/**
	 * @return an ArrayList containing all of the keys seen in the input file for the current FiveC object
	 */
	public ArrayList<String> getAllKeys(){
		Collections.sort(this.allKeys);
		return this.allKeys;
	}/*  end of getAllKeys method  */
	
	
	
	
	/*  start of getAllFragmentPairIndices method  */
	/**
	 * @return an ArrayList containing all of the fragment pairs seen in the input file for the current FiveC object
	 */
	public ArrayList<int[]> getAllFragmentPairIndices(){
		return getAllFragmentPairIndices(-1, -1);
	}/*  end of getAllFragmentPairIndices method  */

	
	
	
	/*  start of getAllFragmentPairIndices method  */
	/**
	 * @param minGap the minimum gap (inclusive) allowed between the two fragment indices (specify a negative value to get all fragment pair indices regardless of gap)
	 * @param maxGap the maximum gap (inclusive) allowed between the two fragment indices (specify a negative value to get all fragment pair indices regardless of gap)
	 * @return an ArrayList containing all of the fragment pairs seen in the input file for the current FiveC object
	 */
	public ArrayList<int[]> getAllFragmentPairIndices(int minGap, int maxGap){
		
		ArrayList<int[]> allFragmentPairIndices = new ArrayList<int[]>();
		
		for(String currentFragmentPairString: this.getAllKeys()){
			String[] substrings = currentFragmentPairString.split("-");
			
			int[] currentFragmentPairIndices = new int[2];
			currentFragmentPairIndices[0] = Integer.parseInt(substrings[0]);
			currentFragmentPairIndices[1] = Integer.parseInt(substrings[1]);
			
			if((minGap >= 0) && (maxGap >= 0)){

				int currentGap = currentFragmentPairIndices[1] - currentFragmentPairIndices[0];
				
				if((currentGap > maxGap) || (currentGap < minGap)){
					continue;
				}
			}
			allFragmentPairIndices.add(currentFragmentPairIndices);
		}
		allFragmentPairIndices.trimToSize();
		
		return allFragmentPairIndices;
	}/*  end of getAllFragmentPairIndices method  */
	
	
	
	
	/*  start of getProbeLength15 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 15bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength15(){
		return this.probeLength15;
	}/*  end of getProbeLength15 method  */

	
	
	
	/*  start of getProbeLength18 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 18bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength18(){
		return this.probeLength18;
	}/*  end of getProbeLength18 method  */

	
	
	
	
	/*  start of getProbeLength19 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 19bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength19(){
		return this.probeLength19;
	}/*  end of getProbeLength19 method  */
	
	
	
	
	/*  start of getProbeLength20 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 20bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength20(){
		return this.probeLength20;
	}/*  end of getProbeLength20 method  */
	
	
	
	
	/*  start of getProbeLength21 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 21bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength21(){
		return this.probeLength21;
	}/*  end of getProbeLength21 method  */
	
	
	
	
	/*  start of getProbeLength22 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 22bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength22(){
		return this.probeLength22;
	}/*  end of getProbeLength22 method  */
	
	
	
	
	/*  start of getProbeLength23 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 23bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength23(){
		return this.probeLength23;
	}/*  end of getProbeLength23 method  */
	
	
	
	
	/*  start of getProbeLength24 method  */
	/**
	 * @return a HashMap containing all of the signal intensity values for the 24bp probe lengths of the current FiveC object
	 */
	public HashMap<String, Double> getProbeLength24(){
		return this.probeLength24;
	}/*  end of getProbeLength24 method  */
	
	
	
	
	/*  start of outputToFile method  */
	/**
	 * @return void
	 */
	public void outputToFile(){
		this.outputToFile(this.getOutputPath() + this.getFilename());
	}/*  end of outputToFile method  */
	
	
	
	
	/*  start of outputToFile method  */
	/**
	 * @param pathFilename a String containing the path and filename for the output file
	 */
	public void outputToFile(String pathFilename){
		if(pathFilename == null){
			System.err.println("Error: the pathFilename is null and the FiveC object cannot be output to file.\nSystem.exit(0)");
			System.exit(0);
		}
		
		
		try{
			BufferedWriter bw = new BufferedWriter(new FileWriter(pathFilename));
			bw.write(this.getDataAsString());
			bw.flush();
			bw.close();
		}catch (IOException e) {
			e.printStackTrace();
		}//end of try-catch block
//		System.out.println("\nOutput file: " + pathFilename);
	}/*  end of outputToFile method  */
	
	
	
	
	/*  start of outputNeighboursOnlyToFile method  */
	/**
	 * @param pathFilename a String containing the path and filename for the output file
	 * @param maxGapInclusive an integer specifying the maximum distance between neighbours (inclusive)
	 */
	public void outputNeighboursOnlyToFile(String pathFilename, int maxGapInclusive){
		if(pathFilename == null){
			System.err.println("Error: the pathFilename is null and the FiveC object cannot be output to file.\nSystem.exit(0)");
			System.exit(0);
		}
		
		
		try{
			BufferedWriter bw = new BufferedWriter(new FileWriter(pathFilename));
			bw.write(this.getDataNeighboursOnlyAsString(maxGapInclusive));
			bw.flush();
			bw.close();
		}catch (IOException e) {
			e.printStackTrace();
		}//end of try-catch block
		
//		System.out.println("\nOutput file: " + pathFilename);
	}/*  end of outputNeighboursOnlyToFile method  */
	
	
	
	
	/*  start of getDataNeighboursOnlyAsString method  */
	/**
	 * @return a String representation of the data (neighbours only) for the current FiveC object
	 * @param maxGapInclusive an integer specifying the maximum distance between neighbours (inclusive)
	 */
	private String getDataNeighboursOnlyAsString(int maxGapInclusive){
		return this.getDataAsString(true, maxGapInclusive);
	}/*  end of getDataNeighboursOnlyAsString method  */
	
	
	
	
	/*  start of getDataAsString method  */
	/**
	 * @return a String representation of the data for the current FiveC object
	 */
	private String getDataAsString(){
		return this.getDataAsString(false, 0);
	}/*  end of getDataAsString method  */
	
	
	
	
	/*  start of getDataAsString method  */
	/**
	 * @param neighboursOnly a boolean indicating to return just the neighbour pairs (true) or all the data (false)
	 * @param maxGapInclusive an integer specifying the maximum distance between neighbours (inclusive), only used if neighboursOnly is set to true
	 * @return a String representation of the data for the current FiveC object
	 */
	private String getDataAsString(boolean neighboursOnly, int maxGapInclusive){
		if(allKeys == null){
			System.err.println("Error: the allKeys object is null and the String representation of the data cannot be generated.\nSystem.exit(0)");
			System.exit(0);
		}
		
		Collections.sort(this.allKeys);
		
		String returnString = "";
		
		for(String currentKey: allKeys){
			
			String index1 = currentKey.substring(0, currentKey.lastIndexOf("-"));
			String index2 = currentKey.substring(currentKey.lastIndexOf("-")+1);
			
			if( (!neighboursOnly) || (Math.abs(Integer.valueOf(index1) - Integer.valueOf(index2)) <= maxGapInclusive) ){

				returnString += index1 + "\t" + index2 + "\t";

				if(this.probeLength15.containsKey(currentKey)){
					returnString += this.probeLength15.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength18.containsKey(currentKey)){
					returnString += this.probeLength18.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength19.containsKey(currentKey)){
					returnString += this.probeLength19.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength20.containsKey(currentKey)){
					returnString += this.probeLength20.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength21.containsKey(currentKey)){
					returnString += this.probeLength21.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength22.containsKey(currentKey)){
					returnString += this.probeLength22.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength23.containsKey(currentKey)){
					returnString += this.probeLength23.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\t";

				if(this.probeLength24.containsKey(currentKey)){
					returnString += this.probeLength24.get(currentKey);
				}
				else{
					returnString += "0";
				}
				returnString += "\n";
			}
		}
		
		
		return returnString;
	}/*  end of getDataAsString method  */
	
	
	
	
	/*  start of toString method  */
	/**
	 * @return a String representation of the FiveC object
	 */
	public String toString(){
		if(this.filename == null){
			System.err.println("Error: the filename is null and the String representation cannot be determined.\nSystem.exit(0)");
			System.exit(0);
		}
		
		return this.filename + "\n" + this.getDataAsString();
	}/*  end of toString method  */
	
	
	
	
	/*  start of getMedianValues method  */
	/**
	 * @return an array containing the median value for each feature length in order (15bp, 18bp, 19bp, 20bp, 21bp, 22bp, 23bp, 24bp)
	 */
	public double[] getMedianValues(){
		
		double[] medianValues = new double[8];
		
		medianValues[0] = this.calculateMedian(this.probeLength15);
		medianValues[1] = this.calculateMedian(this.probeLength18);
		medianValues[2] = this.calculateMedian(this.probeLength19);
		medianValues[3] = this.calculateMedian(this.probeLength20);
		medianValues[4] = this.calculateMedian(this.probeLength21);
		medianValues[5] = this.calculateMedian(this.probeLength22);
		medianValues[6] = this.calculateMedian(this.probeLength23);
		medianValues[7] = this.calculateMedian(this.probeLength24);
		
		return medianValues;
	}/*  end of getMedianValues method  */
	
	
	
	
	/*  start of calculateMedian method  */
	/**
	 * @param signalIntensityHashMap the HashMap containing all of the signal intensity values
	 * @return the median signal value in the input signal intensity hash map
	 */
	private double calculateMedian(HashMap<String, Double> signalIntensityHashMap){
		double median = 0;
		
		ArrayList<Double> values = new ArrayList<Double>(signalIntensityHashMap.values());
		values.trimToSize();
		Collections.sort(values);
		
		if(values.size() > 0){
		
			median = values.get(values.size() / 2);
		
			//  if there are an even number of entries, then the median is the average of two entries
			if((values.size() % 2) == 0){
				int index2 = values.size() / 2;
				int index1 = index2 - 1;
			
				median = (values.get(index1) + values.get(index2)) / (double)(2.0);
			}
		}
		return median;
	}/*  end of calculateMedian method  */
	
	
	
	
	/*  start of getAverageValues method  */
	/**
	 * @return an array containing the average value for each feature length in order (15bp, 18bp, 19bp, 20bp, 21bp, 22bp, 23bp, 24bp)
	 */
	public double[] getAverageValues(){
		
		double[] averageValues = new double[8];
		
		averageValues[0] = this.calculateAverage(this.probeLength15);
		averageValues[1] = this.calculateAverage(this.probeLength18);
		averageValues[2] = this.calculateAverage(this.probeLength19);
		averageValues[3] = this.calculateAverage(this.probeLength20);
		averageValues[4] = this.calculateAverage(this.probeLength21);
		averageValues[5] = this.calculateAverage(this.probeLength22);
		averageValues[6] = this.calculateAverage(this.probeLength23);
		averageValues[7] = this.calculateAverage(this.probeLength24);
		
		return averageValues;
	}/*  end of getAverageValues method  */
	
	
	
	
	/*  start of calculateAverage method  */
	/**
	 * @param signalIntensityHashMap the HashMap containing all of the signal intensity values
	 * @return the average signal value in the input signal intensity hash map
	 */
	private double calculateAverage(HashMap<String, Double> signalIntensityHashMap){
		double average = 0;
		
		ArrayList<Double> values = new ArrayList<Double>(signalIntensityHashMap.values());
		values.trimToSize();
		
		for(double currentValue: values){
			if(currentValue <= 0){
				System.err.println("Error: the value is less than or equal to zero when calculating the average.\nSystem.exit(0)");
				System.exit(0);
			}
			average += currentValue;
		}
		
		if(values.size() > 1){
			average /= (double)values.size();
		}
		return average;
	}/*  end of calculateAverage method  */
	
	
	
	
	/*  start of removeProbes method  */
	/**
	 * @param probePairsToExclude an ArrayList specifying all of the probe pairs to remove from the current FiveC object
	 */
	public void removeProbes(ArrayList<int[]> probePairsToExclude){
		if(probePairsToExclude == null){
			System.err.println("Error: the list of probe pairs to exclude array is null and cannot be removed.\nSystem.exit(0)");
			System.exit(0);
		}
		
		for(int[] currentProbePairToExclude: probePairsToExclude){
			this.removeProbe(currentProbePairToExclude);
		}
	}/*  end of removeProbes method  */
	
	
	
	
	/*  start of removeProbe method  */
	/**
	 * @param probePairToExclude an int array of size two (2) specifying the probe pair to remove from the current FiveC object
	 */
	public void removeProbe(int[] probePairToExclude){
		if(probePairToExclude == null){
			System.err.println("Error: the probe pair to exclude array is null and cannot be removed.\nSystem.exit(0)");
			System.exit(0);
		}
		
		int firstIndex = probePairToExclude[0];
		int secondIndex = probePairToExclude[1];
		
		String forwardKey = firstIndex + "-" + secondIndex;
		String reverseKey = secondIndex + "-" + firstIndex;
		
		if(this.allKeys.contains(forwardKey)){
			this.allKeys.remove(forwardKey);
		}
		
		if(this.allKeys.contains(reverseKey)){
			this.allKeys.remove(reverseKey);
		}
		
		
		if(this.probeLength15.containsKey(forwardKey)){
			this.probeLength15.remove(forwardKey);
		}
		if(this.probeLength15.containsKey(reverseKey)){
			this.probeLength15.remove(reverseKey);
		}
		
		if(this.probeLength18.containsKey(forwardKey)){
			this.probeLength18.remove(forwardKey);
		}
		if(this.probeLength18.containsKey(reverseKey)){
			this.probeLength18.remove(reverseKey);
		}

		if(this.probeLength19.containsKey(forwardKey)){
			this.probeLength19.remove(forwardKey);
		}
		if(this.probeLength19.containsKey(reverseKey)){
			this.probeLength19.remove(reverseKey);
		}

		if(this.probeLength20.containsKey(forwardKey)){
			this.probeLength20.remove(forwardKey);
		}
		if(this.probeLength20.containsKey(reverseKey)){
			this.probeLength20.remove(reverseKey);
		}

		if(this.probeLength21.containsKey(forwardKey)){
			this.probeLength21.remove(forwardKey);
		}
		if(this.probeLength21.containsKey(reverseKey)){
			this.probeLength21.remove(reverseKey);
		}

		if(this.probeLength22.containsKey(forwardKey)){
			this.probeLength22.remove(forwardKey);
		}
		if(this.probeLength22.containsKey(reverseKey)){
			this.probeLength22.remove(reverseKey);
		}

		if(this.probeLength23.containsKey(forwardKey)){
			this.probeLength23.remove(forwardKey);
		}
		if(this.probeLength23.containsKey(reverseKey)){
			this.probeLength23.remove(reverseKey);
		}

		if(this.probeLength24.containsKey(forwardKey)){
			this.probeLength24.remove(forwardKey);
		}
		if(this.probeLength24.containsKey(reverseKey)){
			this.probeLength24.remove(reverseKey);
		}
	}/*  end of removeProbe method  */
	
	
	
	
	/*  start of applySaturationFilter method  */
	/**
	 * @param saturationLimit the maximum value (inclusive) for the signal intensities, anything larger is removed from the current FiveC object
	 */
	public void applySaturationFilter(double saturationLimit){
		
		for(String currentKey: allKeys){
			
			if(this.probeLength15.containsKey(currentKey)){
				if(this.probeLength15.get(currentKey) > saturationLimit){
					this.probeLength15.put(currentKey, 0.0);
					
					if(this.probeLength18.containsKey(currentKey)){
						this.probeLength18.put(currentKey, 0.0);
					}
					
					if(this.probeLength19.containsKey(currentKey)){
						this.probeLength19.put(currentKey, 0.0);
					}
					
					if(this.probeLength20.containsKey(currentKey)){
						this.probeLength20.put(currentKey, 0.0);
					}
					
					if(this.probeLength21.containsKey(currentKey)){
						this.probeLength21.put(currentKey, 0.0);
					}
					
					if(this.probeLength22.containsKey(currentKey)){
						this.probeLength22.put(currentKey, 0.0);
					}
					
					if(this.probeLength23.containsKey(currentKey)){
						this.probeLength23.put(currentKey, 0.0);
					}
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength18.containsKey(currentKey)){
				if(this.probeLength18.get(currentKey) > saturationLimit){
					this.probeLength18.put(currentKey, 0.0);
					
					if(this.probeLength19.containsKey(currentKey)){
						this.probeLength19.put(currentKey, 0.0);
					}
					
					if(this.probeLength20.containsKey(currentKey)){
						this.probeLength20.put(currentKey, 0.0);
					}
					
					if(this.probeLength21.containsKey(currentKey)){
						this.probeLength21.put(currentKey, 0.0);
					}
					
					if(this.probeLength22.containsKey(currentKey)){
						this.probeLength22.put(currentKey, 0.0);
					}
					
					if(this.probeLength23.containsKey(currentKey)){
						this.probeLength23.put(currentKey, 0.0);
					}
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength19.containsKey(currentKey)){
				if(this.probeLength19.get(currentKey) > saturationLimit){
					this.probeLength19.put(currentKey, 0.0);
					
					if(this.probeLength20.containsKey(currentKey)){
						this.probeLength20.put(currentKey, 0.0);
					}
					
					if(this.probeLength21.containsKey(currentKey)){
						this.probeLength21.put(currentKey, 0.0);
					}
					
					if(this.probeLength22.containsKey(currentKey)){
						this.probeLength22.put(currentKey, 0.0);
					}
					
					if(this.probeLength23.containsKey(currentKey)){
						this.probeLength23.put(currentKey, 0.0);
					}
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength20.containsKey(currentKey)){
				if(this.probeLength20.get(currentKey) > saturationLimit){
					this.probeLength20.put(currentKey, 0.0);
					
					if(this.probeLength21.containsKey(currentKey)){
						this.probeLength21.put(currentKey, 0.0);
					}
					
					if(this.probeLength22.containsKey(currentKey)){
						this.probeLength22.put(currentKey, 0.0);
					}
					
					if(this.probeLength23.containsKey(currentKey)){
						this.probeLength23.put(currentKey, 0.0);
					}
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength21.containsKey(currentKey)){
				if(this.probeLength21.get(currentKey) > saturationLimit){
					this.probeLength21.put(currentKey, 0.0);
					
					if(this.probeLength22.containsKey(currentKey)){
						this.probeLength22.put(currentKey, 0.0);
					}
					
					if(this.probeLength23.containsKey(currentKey)){
						this.probeLength23.put(currentKey, 0.0);
					}
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength22.containsKey(currentKey)){
				if(this.probeLength22.get(currentKey) > saturationLimit){
					this.probeLength22.put(currentKey, 0.0);
					
					if(this.probeLength23.containsKey(currentKey)){
						this.probeLength23.put(currentKey, 0.0);
					}
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength23.containsKey(currentKey)){
				if(this.probeLength23.get(currentKey) > saturationLimit){
					this.probeLength23.put(currentKey, 0.0);
					
					if(this.probeLength24.containsKey(currentKey)){
						this.probeLength24.put(currentKey, 0.0);
					}
				}
			}
			
			if(this.probeLength24.containsKey(currentKey)){
				if(this.probeLength24.get(currentKey) > saturationLimit){
					this.probeLength24.put(currentKey, 0.0);
				}
			}
		}
	}/*  end of applySaturationFilter method  */
	
	
	
	
	/*  start of getFragmentPairValues method  */
	/**
	 * @param index1 the index of the first fragment in the pair
	 * @param index2 the index of the second fragment in the pair
	 * @return an array of length eight (8) containing the signal values for the specified fragment pair (15, 18, 19, 20, 21, 22, 23, 24 nt) (if the fragment pair does not exist for the current FiveC experiment, null is returned)
	 */
	public double[] getFragmentPairValues(int index1, int index2){
		
		//  always have the keys in smaller--bigger order
		if(index1 > index2){
			int temp = index2;
			index2 = index1;
			index1 = temp;
		}
		
		//  create the String representation of the current fragment pair to use as a key
		String keyForCurrentPair = index1 + "-" + index2;
		
		
		double[] signalValues = new double[8];
		
		if(this.allKeys.contains(keyForCurrentPair)){
			signalValues[0] = this.probeLength15.get(keyForCurrentPair);
			signalValues[1] = this.probeLength18.get(keyForCurrentPair);
			signalValues[2] = this.probeLength19.get(keyForCurrentPair);
			signalValues[3] = this.probeLength20.get(keyForCurrentPair);
			signalValues[4] = this.probeLength21.get(keyForCurrentPair);
			signalValues[5] = this.probeLength22.get(keyForCurrentPair);
			signalValues[6] = this.probeLength23.get(keyForCurrentPair);
			signalValues[7] = this.probeLength24.get(keyForCurrentPair);
		}
		else{
			return null;
		}
		
		return signalValues;
	}/*  end of getFragmentPairValues method  */
	
	
	
	
	/*  start of getAllFragmentPairValues method  */
	/**
	 * @param fragmentLength the feature length for which to extract the signal values (can be one of 15, 18, 19, 20, 21, 22, 23, or 24 nt)
	 * @param minGap the minimum gap (inclusive) allowed between the two fragment indices
	 * @param maxGap the maximum gap (inclusive) allowed between the two fragment indices
	 * @return a list containing all of the signals for the specified feature length probes that have a gap distance contained in the range [minGap-maxGap]
	 */
	public ArrayList<Double> getAllFragmentPairValues(int fragmentLength, int minGap, int maxGap){
		
		HashMap<String, Double> HashMapOfSignalsToUse = null;
		
		switch (fragmentLength) {
		case 15:
			HashMapOfSignalsToUse = this.getProbeLength15();
			break;
		case 18:
			HashMapOfSignalsToUse = this.getProbeLength18();
			break;
		case 19:
			HashMapOfSignalsToUse = this.getProbeLength19();
			break;
		case 20:
			HashMapOfSignalsToUse = this.getProbeLength20();
			break;
		case 21:
			HashMapOfSignalsToUse = this.getProbeLength21();
			break;
		case 22:
			HashMapOfSignalsToUse = this.getProbeLength22();
			break;
		case 23:
			HashMapOfSignalsToUse = this.getProbeLength23();
			break;
		case 24:
			HashMapOfSignalsToUse = this.getProbeLength24();
			break;
		default:
			System.err.println("Error: need to specify a valid fragment length (15, 18, 19, 20, 21, 22, 23, or 24 nt\nSystem.exit(0)");
			System.exit(0);
			break;
		}
		
		
		ArrayList<Double> signalValues = new ArrayList<Double>();
		
		
		//  go through all of the fragment pairs and select only those that are contained within the [minGap-maxGap] range
		for(String keyForCurrentPair: this.getAllKeys()){
			String[] substrings = keyForCurrentPair.split("-");
			int index1 = Integer.parseInt(substrings[0]);
			int index2 = Integer.parseInt(substrings[1]);
			
			int currentGap = index2 - index1;
			
			if((currentGap <= maxGap) && (currentGap >= minGap)){
				if(HashMapOfSignalsToUse.containsKey(keyForCurrentPair)){
					signalValues.add(HashMapOfSignalsToUse.get(keyForCurrentPair));
//					System.out.println("including signal for fragment pair: " + keyForCurrentPair);
				}
			}
		}
		signalValues.trimToSize();
		
		return signalValues;
	}/*  end of getAllFragmentPairValues method  */
	
	
	
	
}/*  end of FiveC class  */
