package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;


/** ------------------------------------------------------------------------------------------------------------------
 * start of IFcorrelation class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class IFcorrelation {


	
	
	/*  global parameters  */
	private boolean verbose = true;
	private String inputPath = "";
	private String outputPath = "";
	/*  end of global parameters  */
	
	
	
	
	/*  start of constructor method  */
	/**
	 * 
	 */
	public IFcorrelation(){
		
	}/*  end of constructor method  */
	
	
	
	
	/*  start of constructor method  */
	/**
	 * @param inputPath
	 * @param outputPath
	 */
	public IFcorrelation(String inputPath, String outputPath){
		this.setInputPath(inputPath);
		this.setOutputPath(outputPath);
	}/*  end of constructor method  */
	
	
	
	
	/*  start of setInputPath method  */
	/**
	 * @param inputPath The String to set as the inputPath
	 */
	public void setInputPath(String inputPath) {
		if(inputPath != null){
			this.inputPath = inputPath;
		}
	}/*  end of setInputPath method  */

	
	
	
	/*  start of setOutputPath method  */
	/**
	 * @param outputPath The String to set as the outputPath
	 */
	public void setOutputPath(String outputPath) {
		if(outputPath != null){
			this.outputPath = outputPath;
		}
	}/*  end of setOutputPath method  */
	
	
	
	
	/*  start of findMaxIndexInFile method  */
	/**
	 * @param filename The name of the IF file to find the maximum index in
	 * @return The maximum index value found in the specified file
	 */
	private int findMaxIndexInFile(String filename){

		int maxIndex = Integer.MIN_VALUE;
		
		String line = null;
		try{
			BufferedReader br = new BufferedReader(new FileReader(this.inputPath + filename));			
			while((line = br.readLine()) != null){
				String values[] = line.split("\t");
				//  make sure that there are exactly five (5) entries in each row
				if(values.length == 5){
					
					int forwardIndex = Integer.valueOf(values[0]);
					int reverseIndex = Integer.valueOf(values[1]);
					
					if(forwardIndex > maxIndex){
						maxIndex = forwardIndex;
					}
					
					if(reverseIndex > maxIndex){
						maxIndex = reverseIndex;
					}
					
				}
				else{
					System.err.println("Error: there are not five (5) entries in each row of the file \"" + this.inputPath + filename + "\"\nSystem.exit(0)");
					System.exit(0);
				}
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
			System.err.println("Error: cannot find file with name \"" + this.inputPath + filename + "\"\nSystem.exit(0)");
			System.exit(0);
		}
		
		if(verbose){
			System.out.println("\tOpened and found maximum index (" + maxIndex + ") in file: " + this.inputPath + filename);
		}
		
		return maxIndex;
	}/*  end of findMaxIndexInFile method  */
	
	
	
	
	/*  start of findMaxIndex method  */
	/**
	 * 
	 */
	private int findMaxIndex(String filename1, String filename2){
		int maxIndexFile1 = this.findMaxIndexInFile(filename1);
		int maxIndexFile2 = this.findMaxIndexInFile(filename2);
		
		
		//  return the larger of the two maximum indices
		if(maxIndexFile1 > maxIndexFile2){
			return maxIndexFile1;
		}
		else{
			return maxIndexFile2;
		}
	}/*  end of findMaxIndex method  */
	
	
	
	
	/*  start of loadIFvalues method  */
	/**
	 * 
	 */
	private void loadIFvalues(String filename, double[][] IFsArray){
		

		String line = null;
		try{
			BufferedReader br = new BufferedReader(new FileReader(this.inputPath + filename));			
			while((line = br.readLine()) != null){
				String values[] = line.split("\t");
				//  make sure that there are exactly five (5) entries in each row
				if(values.length == 5){
					
					int forwardIndex = Integer.valueOf(values[0]);
					int reverseIndex = Integer.valueOf(values[1]);
					double IFvalue = Double.valueOf(values[2]);
					
					//  make sure that the forward index is smaller than the reverse index
					if(reverseIndex < forwardIndex){
						int tempIndex = reverseIndex;
						reverseIndex = forwardIndex;
						forwardIndex = tempIndex;
					}
					if(forwardIndex == reverseIndex){
						System.err.println("Error: the forward and reverse indices are the same.\nSystem.exit(0)");
						System.exit(0);
					}
					
					
					//  store the IFs value in the array with the fragment indices as the indices into the array
					//  only store them in the upper diagonal of the square matrix (i.e. forwardIndex must be less than reverseIndex
					if(IFsArray[forwardIndex][reverseIndex] == -1){
						IFsArray[forwardIndex][reverseIndex] = IFvalue;
					}
					else{
						System.err.println("Error: there is already a value stored for index pair " + forwardIndex + "-" + reverseIndex + " in file " + this.inputPath + filename);
						System.err.println("System.exit(0)");
						System.exit(0);
					}
					
					
				}
				else{
					System.err.println("Error: there are not five (5) entries in each row of the file \"" + this.inputPath + filename + "\"\nSystem.exit(0)");
					System.exit(0);
				}
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
			System.err.println("Error: cannot find file with name \"" + this.inputPath + filename + "\"\nSystem.exit(0)");
			System.exit(0);
		}
		
		if(verbose){
			System.out.println("\tOpened and loaded IF values from file: " + this.inputPath + filename);
		}
		
		
	}/*  end of loadIFvalues method  */
	
	
	
	
	/*  start of outputIFsIntersection method  */
	/**
	 * 
	 */
	private void outputIFsIntersection(String outputFilename, double[][] IFsFile1, double[][] IFsFile2, String IFsFilename1, String IFsFilename2){
		
		
		try{
			BufferedWriter bw = new BufferedWriter(new FileWriter(this.outputPath + outputFilename));

			//  write the first line
			bw.write("index1\tindex2\t" + IFsFilename1 + "[x]\t" + IFsFilename2 + "[y]");
			bw.newLine();
			bw.flush();
			
			//  write all of the values to file
			for(int index1=1; index1<IFsFile1.length; index1++){
				for(int index2=index1+1; index2<IFsFile1.length; index2++){
					if((IFsFile1[index1][index2] != -1) && (IFsFile2[index1][index2] != -1)){
						bw.write(index1 + "\t" + index2 + "\t" + IFsFile1[index1][index2] + "\t" + IFsFile2[index1][index2]);
						bw.newLine();
						bw.flush();
					}
				}
			}
			bw.close();
		}catch (IOException e) {
			e.printStackTrace();
		}//end of try-catch block
	
		if(verbose){
			System.out.println("\nOutput file: " + this.outputPath + outputFilename);
		}
		
		
	}/*  end of outputIFsIntersection method  */
	
	
	
	
	/*  start of generateCorrelationFile method  */
	/**
	 * 
	 */
	public void generateCorrelationFile(String filename1, String filename2){
		if((filename1 == null) || (filename2 == null)){
			System.err.println("Error: the filenames must both be non-null to generate the correlation output file.");
			System.err.println("System.exit(0)");
			System.exit(0);
		}
		
		
		//  generate the output filename
		//  the first column is referred to as 'x'
		//  the second column is referred to as 'y'
		String outputFilename = filename1.substring(0, filename1.lastIndexOf('.')) + "[x]__" + filename2.substring(0, filename2.lastIndexOf('.')) + "[y].txt"; 
		
		
		
		//  find the maximum index across the two files
		int maxIndex = this.findMaxIndex(filename1, filename2);
		
		
		double[][] IFsFile1 = new double[maxIndex+1][maxIndex+1];	//  need to make the size 1 bigger than the maxIndex to be
		double[][] IFsFile2 = new double[maxIndex+1][maxIndex+1];	//  able to index into the array using the IF indices
		//  start the arrays containing all values of -1
		for(int i=0; i<=maxIndex; i++){
			for(int j=0; j<=maxIndex; j++){
				IFsFile1[i][j] = -1;
				IFsFile2[i][j] = -1;
			}
		}
		
		
		this.loadIFvalues(filename1, IFsFile1);
		this.loadIFvalues(filename2, IFsFile2);
		
		
		this.outputIFsIntersection(outputFilename, IFsFile1, IFsFile2, filename1, filename2);
		
		
		if(verbose){
			System.out.println("Done!");
		}
	}/*  end of generateCorrelationFile method  */
	
	
	
	
	
}/*  end of setInputPath class  */
