package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;


/** ------------------------------------------------------------------------------------------------------------------
 * start of LOOCVresults class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class LOOCVresults {
	
	
	
	
	/*  global parameters  */
	private String relation = "";
	private int numInstances;
	private int numAttributes;
	private int numClasses;
	private double[][] confusionMatrix;
	private ArrayList<ArrayList<String>> resultsList;
	private ArrayList<String> correctlyPredictedList;
	private ArrayList<String> incorrectlyPredictedList;
	/*  end of global parameters  */
	
	
	

	/*  start of getRelation method  */
	/**
	 * @return the relation
	 */
	public String getRelation() {
		return relation;
	}/*  end of getRelation method  */
	
	
	
	
	/*  start of setRelation method  */
	/**
	 * @param relation the relation to set
	 */
	public void setRelation(String relation) {
		if(relation == null){
			System.err.println("Error: the relation is null and cannot be set.\nSystem.exit(0)");
			System.exit(0);
		}
		this.relation = relation;
	}/*  end of setRelation method  */
	
	
	
	
	/*  start of getNumInstances method  */
	/**
	 * @return the numInstances
	 */
	public int getNumInstances() {
		return numInstances;
	}/*  end of getNumInstances method  */




	/*  start of setNumInstances method  */
	/**
	 * @param numInstances the numInstances to set
	 */
	public void setNumInstances(int numInstances) {
		this.numInstances = numInstances;
	}/*  end of setNumInstances method  */




	/*  start of getNumAttributes method  */
	/**
	 * @return the numAttributes
	 */
	public int getNumAttributes() {
		return numAttributes;
	}/*  end of getNumAttributes method  */




	/*  start of setNumAttributes method  */
	/**
	 * @param numAttributes the numAttributes to set
	 */
	public void setNumAttributes(int numAttributes) {
		this.numAttributes = numAttributes;
	}/*  end of setNumAttributes method  */




	/*  start of getNumClasses method  */
	/**
	 * @return the numClasses
	 */
	public int getNumClasses() {
		return numClasses;
	}/*  end of getNumClasses method  */




	/*  start of setNumClasses method  */
	/**
	 * @param numClasses the numClasses to set
	 */
	public void setNumClasses(int numClasses) {
		this.numClasses = numClasses;
	}/*  end of setNumClasses method  */




	/*  start of getConfusionMatrix method  */
	/**
	 * @return the confusionMatrix
	 */
	public double[][] getConfusionMatrix() {
		return confusionMatrix;
	}/*  end of getConfusionMatrix method  */




	/*  start of setConfusionMatrix method  */
	/**
	 * @param confusionMatrix the confusionMatrix to set
	 */
	public void setConfusionMatrix(double[][] confusionMatrix) {
		if(confusionMatrix == null){
			System.err.println("Error: the confusion matrix argument is null and cannot be set.\nSystem.exit(0)");
			System.exit(0);
		}
		this.confusionMatrix = confusionMatrix;
	}/*  end of setConfusionMatrix method  */
	
	
	
	
	/*  start of getCorrectlyPredictedList method  */
	/**
	 * @return the correctlyPredictedList
	 */
	public ArrayList<String> getCorrectlyPredictedList() {
		if(this.correctlyPredictedList == null){
			this.correctlyPredictedList = new ArrayList<String>();
		}
		return this.correctlyPredictedList;
	}/*  end of getCorrectlyPredictedList method  */




	/*  start of getIncorrectlyPredictedList method  */
	/**
	 * @return the incorrectlyPredictedList
	 */
	public ArrayList<String> getIncorrectlyPredictedList() {
		if(this.incorrectlyPredictedList == null){
			this.incorrectlyPredictedList = new ArrayList<String>();
		}
		return this.incorrectlyPredictedList;
	}/*  end of getIncorrectlyPredictedList method  */




	/*  start of addResult method  */
	/**
	 * @param experimentLeftOutName the name of the left out experiment (the test dataset)
	 * @param confusionMatrix the confusion matrix resulting from the classifier evaluation of the test dataset
	 */
	public void addResult(String experimentLeftOutName, String confusionMatrix){
		ArrayList<String> newResultEntry = new ArrayList<String>();
		newResultEntry.add(experimentLeftOutName);
		newResultEntry.add(confusionMatrix);
		newResultEntry.trimToSize();
		
		this.addEntryToResultsList(newResultEntry);
	}/*  end of addResult method  */
	
	
	
	
	/*  start of addEntryToResultsList method  */
	/**
	 * @param newResultEntry the results entry to add to the list of results entries
	 */
	private void addEntryToResultsList(ArrayList<String> newResultEntry){
		if(this.resultsList == null){
			this.resultsList = new ArrayList<ArrayList<String>>();
		}
		this.resultsList.add(newResultEntry);
		this.resultsList.trimToSize();
	}/*  end of addEntryToResultsList method  */
	
	
	

	/*  start of setCompleteDatasetProperties method  */
	/**
	 * @param numInstances the number of instances defined in the complete dataset
	 * @param numAttributes the number of attributes defined in the complete dataset
	 * @param numClasses the total number of class labels defined in the complete dataset
	 */
	public void setCompleteDatasetProperties(int numInstances, int numAttributes, int numClasses){
		this.setNumInstances(numInstances);
		this.setNumAttributes(numAttributes);
		this.setNumClasses(numClasses);
	}/*  end of setCompleteDatasetProperties method  */
	
	
	
	
	/*  start of addTestDatasetToCorrectlyPredictedList method  */
	/**
	 * @param testDatasetName the name of the experiment used as the test dataset that should be added to the list of correctly predicted samples
	 */
	public void addTestDatasetToCorrectlyPredictedList(String testDatasetName){
		if(testDatasetName == null){
			System.err.println("Error: the experiment name is null and cannot be added to the correctly predicted list.\nSystem.exit(0)");
			System.exit(0);
		}
		if(this.correctlyPredictedList == null){
			this.correctlyPredictedList = new ArrayList<String>();
		}
		this.correctlyPredictedList.add(testDatasetName);
	}/*  end of addTestDatasetToCorrectlyPredictedList method  */
	
	
	
	
	/*  start of addTestDatasetToIncorrectlyPredictedList method  */
	/**
	 * @param testDatasetName the name of the experiment used as the test dataset that should be added to the list of incorrectly predicted samples
	 */
	public void addTestDatasetToIncorrectlyPredictedList(String testDatasetName){
		if(testDatasetName == null){
			System.err.println("Error: the experiment name is null and cannot be added to the incorrectly predicted list.\nSystem.exit(0)");
			System.exit(0);
		}
		if(this.incorrectlyPredictedList == null){
			this.incorrectlyPredictedList = new ArrayList<String>();
		}
		this.incorrectlyPredictedList.add(testDatasetName);
	}/*  end of addTestDatasetToIncorrectlyPredictedList method  */
	
	
	
	
	/*  start of constructor  */
	/**
	 * 
	 */
	public LOOCVresults() {
		
	}/*  end of constructor  */
	
	
	
	
	/*  start of outputToFile method  */
	/**
	 * @param output_filename_prefix the prefix to use for the file written to disk (if it is known, then the relation name is appended)
	 */
	public void outputToFile(String output_filename_prefix){
		this.outputToFile(output_filename_prefix, false);
	}/*  end of outputToFile method  */
	
	
		
	
	/*  start of outputToFile method  */
	/**
	 * @param output_filename_prefix the prefix to use for the file written to disk (if it is known, then the relation name is appended)
	 * @param verbose a boolean flag indicating whether to output information to the console (true), or to remain silent (false)
	 */
	public void outputToFile(String output_filename_prefix, boolean verbose){
		if(output_filename_prefix == null){
			System.err.println("Error: the output filename is null and the results cannot be output.\nSystem.exit(0)");
			System.exit(0);
		}
		
		String output_filename = output_filename_prefix + this.getRelation() + ".txt";
		

		try{
			BufferedWriter bw = new BufferedWriter(new FileWriter(output_filename));
			
			
			//  write the details about the complete dataset
			bw.write("==========  Complete dataset details  ==========");
			bw.newLine();
			bw.write("Relation:\t" + this.getRelation());
			bw.newLine();
			bw.write("Number of Attributes:\t" + this.getNumAttributes());
			bw.newLine();
			bw.write("Number of Instances:\t" + this.getNumInstances());
			bw.newLine();
			bw.write("Number of Classes:\t" + this.getNumClasses());
			bw.newLine();
			for(int i=0; i<48; i++){
				bw.write("=");
			}
			bw.newLine();
			bw.flush();
			
			
			//  skip four lines
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.flush();
			
			
			//  write the summary confusion matrix
			bw.write("==========  Summary confusion matrix  ==========");
			bw.newLine();
			bw.flush();
			for(int i=0; i<this.getConfusionMatrix().length; i++){
				bw.write("\t\t");
				for(int j=0; j<this.getConfusionMatrix().length; j++){
					bw.write((int)this.getConfusionMatrix()[i][j] + "\t");
				}
				bw.newLine();
				bw.flush();
			}
			for(int i=0; i<48; i++){
				bw.write("=");
			}
			bw.newLine();
			bw.flush();
			

			//  skip four lines
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.flush();
			
			
			//  write the summary performance statistics of the classifier
			NumberFormat formatterTwoDecimals = new DecimalFormat("0.00");
			
			int numCorrectlyPredicted = 0;
			int numIncorrectlyPredicted = 0;
			for(int i=0; i<this.getConfusionMatrix().length; i++){
				for(int j=0; j<this.getConfusionMatrix().length; j++){
					if(i == j){
						numCorrectlyPredicted += this.getConfusionMatrix()[i][j];
					}
					else{
						numIncorrectlyPredicted += this.getConfusionMatrix()[i][j];
					}
				}
			}
			double percentageCorrect = numCorrectlyPredicted * 100.0 / (double)(numCorrectlyPredicted + numIncorrectlyPredicted);
			double percentageIncorrect = numIncorrectlyPredicted * 100.0 / (double)(numCorrectlyPredicted + numIncorrectlyPredicted);
			
			bw.write("==========  Performance of classifier  ==========");
			bw.newLine();
			bw.flush();
			bw.write("Number correctly predicted:\t" + numCorrectlyPredicted);
			bw.newLine();
			bw.flush();
			bw.write("Number incorrectly predicted:\t" + numIncorrectlyPredicted);
			bw.newLine();
			bw.newLine();
			bw.flush();
			bw.write("Percentage correctly predicted:\t" + formatterTwoDecimals.format(percentageCorrect) + "%");
			bw.newLine();
			bw.flush();
			bw.write("Percentage incorrectly predicted:\t" + formatterTwoDecimals.format(percentageIncorrect) + "%");
			bw.newLine();
			bw.flush();

			for(int i=0; i<49; i++){
				bw.write("=");
			}
			bw.newLine();
			bw.flush();


			//  skip four lines
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.flush();
			
			
			//  write the list of correctly predicted test cases
			bw.write("==========  Correctly predicted test cases  ==========");
			bw.newLine();
			bw.flush();
			for(String currentCorrectlyPredictedTestCaseName: this.getCorrectlyPredictedList()){
				bw.write("\t\t" + currentCorrectlyPredictedTestCaseName);
				bw.newLine();
				bw.flush();
			}
			for(int i=0; i<54; i++){
				bw.write("=");
			}
			bw.newLine();
			bw.flush();
			

			//  skip three lines
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.flush();
			
			
			//  write the list of incorrectly predicted test cases
			bw.write("==========  Incorrectly predicted test cases  ==========");
			bw.newLine();
			bw.flush();
			for(String currentIncorrectlyPredictedTestCaseName: this.getIncorrectlyPredictedList()){
				bw.write("\t\t" + currentIncorrectlyPredictedTestCaseName);
				bw.newLine();
				bw.flush();
			}
			for(int i=0; i<56; i++){
				bw.write("=");
			}
			bw.newLine();
			bw.flush();
			
			
			//  skip four lines
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.newLine();
			bw.flush();
			
			
			//  write the LOOCV results for each test case
			for(ArrayList<String> currentResultEntry: this.resultsList){
				bw.write("==========  Test case \"" + currentResultEntry.get(0) + "\"  ==========");
				bw.newLine();
				bw.flush();
				
				//  write the confusion matrix for the current results entry
				bw.write(currentResultEntry.get(1));
				bw.flush();
				
				for(int i=0; i<(36+currentResultEntry.get(0).length()); i++){
					bw.write("=");
				}
				bw.newLine();
				bw.flush();
				
				
				//  skip three lines
				bw.newLine();
				bw.newLine();
				bw.newLine();
				bw.flush();
			}
			
			
			
			bw.flush();
			bw.close();
		}catch (IOException e) {
			e.printStackTrace();
		}//end of try-catch block
		
		
		if(verbose){
			System.out.println("\n\tOutput file:\t\"" + output_filename + "\".");
		}
	}/*  end of outputToFile method  */
	
	
	
	
	
}/*  end of LOOCVresults class  */
