package modeling;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


/** ------------------------------------------------------------------------------------------------------------------
 * start of MCMC5CMain class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class MCMC5CMain {

	
	/*  start of main method  */
	public static void main(String[] args) {	
		
		
		
		
		boolean verbose = true;
		boolean usingCommandLine = false;
		
		String inputPath = "";
		String outputPath = "";
		
		
		//  load the parameters from the command line
		switch(args.length){
		case 12:
			//  this is the correct number of command line parameters, can use the args String array directly

			usingCommandLine = true;
			verbose = false;

			break;
		case 14:
			//  this is the unpublished number of command line parameters that also sets the input and output paths
			
			inputPath = args[12];
			outputPath = args[13];
			
			usingCommandLine = true;
			verbose = false;
			
			String[] newArgs = new String[12];
			for(int i=0; i<12; i++){
				newArgs[i] = args[i];
			}
			args = newArgs;
			
			break;
		default:
			System.err.println("Error: there need to be exactly twelve (12) command line parameters.\n" +
					"\t\tIFs_filename.txt\n" +
					"\t\tfragments_filename.txt\n" +
					"\t\trun_length\n" +
					"\t\tnumber_of_structures_to_output\n" +
					"\t\tstep_size\n" +
					"\t\texponent\n" +
					"\t\tinitial_structure_case\n" +
					"\t\t\tcase 0: use an initial structure from file as specified by the initial_structure_filename\n" +			
					"\t\t\tcase 1: use a small cube with side length [0, 0.01[\n" +
					"\t\t\tcase 2: use a cube with side length = 10 * avgIF^exponent\n" +
					"\t\t\tcase 3: use a spiral with small random noise\n" +
					"\t\t\tcase 4: use a straight line with small random noise\n" +
					"\t\t\tcase 5: use a circle with no noise\n" +
					"\t\t\tcase 6: use a circle with some noise\n" +
					"\t\tinitial_structure_filename\n" +
					"\t\tsingle_point_translation_move_fraction\n" +
					"\t\tblock_translation_move_fraction\n" +
					"\t\tmirror_move_fraction\n" +
					"\t\tmaximum_neighbour_distance\n" +
					"System.exit(0)");
//			System.exit(0);
			
			break;
		}
		
		
		
		
		/******************************************************
		 * 													  *
		 * 				SET THE PARAMETERS HERE				  *
		 * 													  *
		 ******************************************************/
		
		
		if(!usingCommandLine){
			
			
			inputPath = "MCMC5C/Input/";
			outputPath = "MCMC5C/Output/";
			
			
			args = new String[12];
			
//			args[0] = "IFs_0h_technical.txt";					//	IFs filename
//			args[0] = "IFs_0h_biological2.txt";
//			args[0] = "IFs_4h_technical_3.txt";
//			args[0] = "IFs_4h_biological2.txt";
//			args[0] = "IFs_12h_technical.txt";
//			args[0] = "IFs_12h_biological2.txt";
//			args[0] = "IFs_48h_technical.txt";
//			args[0] = "IFs_48h_biological2.txt";
//			args[0] = "IFs_96h_technical.txt";
//			args[0] = "IFs_96h_biological2.txt";
			
//			args[0] = "IFs_GD_0h_biological2.txt";
//			args[0] = "IFs_GD_0h_technical.txt";
//			args[0] = "IFs_GD_4h_biological2.txt";
//			args[0] = "IFs_GD_4h_technical_3.txt";
//			args[0] = "IFs_GD_12h_biological2.txt";
//			args[0] = "IFs_GD_12h_technical.txt";
//			args[0] = "IFs_GD_48h_biological2.txt";
//			args[0] = "IFs_GD_48h_technical.txt";
//			args[0] = "IFs_GD_96h_biological2.txt";
			args[0] = "IFs_GD_96h_technical.txt";
			
//			args[1] = "fragments_no48_hg19.txt";				//	fragments filename
			args[1] = "fragments_GD.txt";
			args[2] = String.valueOf((long)50000000);				//	run length
			args[3] = String.valueOf(250);						//	number of structures to output
			args[4] = String.valueOf(0.05);						//	step size
			args[5] = String.valueOf(2.0);						//	exponent
			args[6] = String.valueOf(0);						//	initial structure case		//  put back to 6 after ('0' = from file)
//			args[7] = "initial_structure_0hr[4].pdb";			//	initial structure filename
//			args[7] = "initial_structure_4hr[2][1].pdb";
//			args[7] = "initial_structure_12hr[2].pdb";
//			args[7] = "initial_structure_48hr[3].pdb";
//			args[7] = "initial_structure_96hr[1].pdb";
			
//			args[7] = "best_structure_IFs_GD_0Hr.pdb";
//			args[7] = "best_structure_IFs_GD_4Hr[2].pdb";
//			args[7] = "best_structure_IFs_GD_12Hr.pdb";
//			args[7] = "best_structure_IFs_GD_48Hr.pdb";
			args[7] = "best_structure_IFs_GD_96Hr.pdb";
			
			args[8] = String.valueOf(0.8);						//	single point translation fraction
			args[9] = String.valueOf(0.1);						//  block translation fraction
			args[10] = String.valueOf(0.1);						//	mirror fraction
			args[11] = String.valueOf(5);						//  maximum distance between neighbours
			
			
//			verbose = false;
			
			
		}
		

		/******************************************************
		 * 													  *
		 * 				END OF SETTING THE PARAMETERS		  *
		 * 													  * 
		 ******************************************************/
		
		
		
		
		//  create and run the MCMC5C program
		MCMC5C structureSolver = new MCMC5C(inputPath, outputPath, verbose);
		structureSolver.run(args);
		
		
		if(verbose){
			System.out.println("Done!");
		}
	}/*  end of main method  */

	
}/*  end of MCMC5CMain class  */