package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;


/** ------------------------------------------------------------------------------------------------------------------
 * start of NeighboursOnly class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class NeighboursOnly {
	
	
	
	
	/*  global parameters  */
	private static boolean verbose = true;
	private static String inputPath = "";
	private static String outputPath = "";
	/*  end of global parameters  */
	
	

	
	/*  start of getFilenames method  */
	/**
	 * @param filename_experiment_list the file containing the list of experiment matched filenames (normalization region and experimental region)
	 * @return a list of Strings specifying the experiment filenames
	 */
	private static ArrayList<String> getFilenames(String filename_experiment_list){
		if(filename_experiment_list == null){
			System.err.println("Error: the filename list is null.\nSystem.exit(0)");
			System.exit(0);
		}
		
		ArrayList<String> experimentList = new ArrayList<String>();

		String line = null;
		int lineCounter = -1;			//  there is one header line
		try{
			BufferedReader br = new BufferedReader(new FileReader(inputPath + filename_experiment_list));
			while((line = br.readLine()) != null){
				if(lineCounter >= 0){
					experimentList.add(line);
				}
				else{
					//  skip and the header line
//					System.out.println(line);
				}
				lineCounter++;
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
		}
		experimentList.trimToSize();
		
		if(verbose){
			System.out.println("Loaded input file: " + inputPath + filename_experiment_list);
		}
		
		return experimentList;
	}/*  end of getFilenames method  */
	
	
	
	
	/*  start of displayFilenames method  */
	/**
	 * @param experimentList the list of experiment filenames to display
	 */
	private static void displayFilenames(ArrayList<String> experimentList){
		if(experimentList == null){
			System.err.println("Error: the experiment list is null and cannot be displayed.\nSystem.exit(0)");
			System.exit(0);	
		}
		
		System.out.println("Filename");
		for(String currentFilename: experimentList){
			System.out.println(currentFilename);
		}
	}/*  end of displayFilenames method  */
	
	
	
	
	/*  start of filterNeighboursOnly method  */
	/**
	 * @param maxGapInclusive an integer specifying the maximum distance between neighbours (inclusive)
	 * @param experimentList the list of experiment filenames to load and filter for neighbours only
	 */
	private static void filterNeighboursOnly(int maxGapInclusive, ArrayList<String> experimentList){
		if(experimentList == null){
			System.err.println("Error: the experiment list is null and cannot be filtered for neighbours only.\nSystem.exit(0)");
			System.exit(0);
		}
		
		
		for(String currentFilename: experimentList){
			FiveC currentFiveC = new FiveC(currentFilename, inputPath, outputPath);
			currentFiveC.outputNeighboursOnlyToFile(outputPath + currentFiveC.getFilename(), maxGapInclusive);
		}
	}/*  end of filterNeighboursOnly method  */
	
	
	
	
	/*  start of main method  */
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		

		boolean usingCommandLine = false;
		
		int neighboursMaxGap = 3;
		String filename_experiment_list = "default.txt";
		ArrayList<String> experimentList = new ArrayList<String>();
		
		
		//  load the parameters from the command line
		if(args.length >= 2){
			//  first parameter is the maximum distance between neighbours
			neighboursMaxGap = Integer.parseInt(args[0]);
			
			//  all subsequent parameters are the filenames of the experiment files
			for(int i=1; i<args.length; i++){
				experimentList.add(args[i]);
			}
			experimentList.trimToSize();
			
			usingCommandLine = true;
			verbose = false;
		}
		else{
			System.err.println("Error: there need to be at least two (2) command line parameters.\n" +
					"\t\tmaximum_gap_between_neighbours\n" +
					"\t\texperiment_1_filename.txt\n" +
					"\t\texperiment_2_filename.txt\n" +
					"\t\texperiment_..._filename.txt\n" +
					"System.exit(0)");
//			System.exit(0);
		}
		
		
		
		/******************************************************
		 * 													  *
		 * 				SET THE PARAMETERS HERE				  *
		 * 													  *
		 ******************************************************/
		
		
		if(!usingCommandLine){
			
			
			inputPath = "NeighboursOnly/Input/";
			outputPath = "NeighboursOnly/Output/";
			
			
			neighboursMaxGap = 3;
			
			
			filename_experiment_list = "experiment_list.txt";
			
			
		}
		
		
		/******************************************************
		 * 													  *
		 * 				END OF SETTING THE PARAMETERS		  *
		 * 													  * 
		 ******************************************************/
		
		
		
		
		if(!usingCommandLine){
			experimentList = getFilenames(filename_experiment_list);
		}
//		displayFilenames(experimentList);
		
		
		filterNeighboursOnly(neighboursMaxGap, experimentList);
		
		
		if(verbose){
			System.out.println("Done!");
		}
	}/*  end of main method  */

}/*  end of NeighboursOnly class  */
