package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;


/** ------------------------------------------------------------------------------------------------------------------
 * start of NonNeighboursOnly class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class NonNeighboursOnly {
	
	
	
	
	/*  global parameters  */
	private static boolean verbose = true;
	private static String inputPath = "";
	private static String outputPath = "";
	/*  end of global parameters  */
	
	
	
	
	/*  start of filterData method  */
	/**
	 * @param filename the input filename
	 * @param minNeighbourGap the minimum gap between neighbours to retain and output (needs to be strictly greater than minimum gap)
	 */
	private static void filterData(String filenameInput, int minNeighbourGap){
		if(filenameInput == null){
			System.err.println("Error: the filename is null and the SVM weights cannot be loaded from file.\nSystem.exit(0)");
			System.exit(0);
		}
		
//		String filenameOutput = filenameInput.subSequence(0, filenameInput.lastIndexOf(".")) + "_>" + minNeighbourGap + ".txt";
		String filenameOutput = filenameInput;
		
		int lineCounter = 0;		//  change to -1 if there is a header line ('0' for no header line)
		String line = null;
		try{
			BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath + filenameOutput));
			
			BufferedReader br = new BufferedReader(new FileReader(inputPath + filenameInput));			
			while((line = br.readLine()) != null){
				if(lineCounter >= 0){		//  there is one header line to skip
					String values[] = line.split("\t");
					//  make sure that there are exactly five (5) entries in each row
					if(values.length == 5){
						
						int indexFeature1 = Integer.parseInt(values[0]);
						int indexFeature2 = Integer.parseInt(values[1]);
						
						int neighbourGap = Math.abs(indexFeature2 - indexFeature1);
						
						if(neighbourGap > minNeighbourGap){
							System.out.println(line);
							bw.write(line);
							bw.newLine();
							bw.flush();
						}
					}
					else{
						System.err.println("Error: there are not five (5) entries in each row of the file \"" + inputPath + filenameInput + "\"\nSystem.exit(0)");
						System.exit(0);
					}
				}
				else{
					//  display and output the header line
					System.out.println(line);
					bw.write(line);
					bw.newLine();
					bw.flush();
				}
				lineCounter++;
			}
			bw.flush();
			bw.close();
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
			System.err.println("Error: cannot find file with name \"" + inputPath + filenameInput + "\"\nSystem.exit(0)");
			System.exit(0);
		}
		
		if(verbose){
			System.out.println("\n\tOpened and read file: " + inputPath + filenameInput);
			System.out.println("\tOutput file: " + outputPath + filenameOutput);
		}
	}/*  end of filterData method  */
	
	
	
	
	/*  start of main method  */
	/**
	 * @param args
	 */
	public static void main(String[] args) {

		
		ArrayList<String> input_filenames = new ArrayList<String>();
		
		
		
		
		/******************************************************
		 * 													  *
		 * 				SET THE PARAMETERS HERE				  *
		 * 													  *
		 ******************************************************/
		
		
		inputPath = "NonNeighboursOnly/Input/";
		outputPath = "NonNeighboursOnly/Output/";
		
		
		input_filenames.add("IFs_0h_biological.txt");
		input_filenames.add("IFs_0h_original.txt");
		input_filenames.add("IFs_0h_technical.txt");
		input_filenames.add("IFs_4h_biological.txt");
		input_filenames.add("IFs_4h_original_1.txt");
		input_filenames.add("IFs_4h_original_2.txt");
		input_filenames.add("IFs_4h_technical_1.txt");
		input_filenames.add("IFs_4h_technical_2.txt");
		input_filenames.add("IFs_4h_technical_3.txt");
		input_filenames.add("IFs_12h_biological.txt");
		input_filenames.add("IFs_12h_original.txt");
		input_filenames.add("IFs_12h_technical.txt");
		input_filenames.add("IFs_48h_biological.txt");
		input_filenames.add("IFs_48h_original.txt");
		input_filenames.add("IFs_48h_technical.txt");
		input_filenames.add("IFs_96h_biological.txt");
		input_filenames.add("IFs_96h_original.txt");
		input_filenames.add("IFs_96h_technical.txt");
		
		
		int minimumGap = 7;				//  note the neighbours must be exclusively greater than the minimum gap value to be retained


		/******************************************************
		 * 													  *
		 * 				END OF SETTING THE PARAMETERS		  *
		 * 													  * 
		 ******************************************************/
		
		
		
		
		for(String input_filename: input_filenames){
			filterData(input_filename, minimumGap);
		}
		
		
		if(verbose){
			System.out.println("Done!");
		}
		
	}/*  end of main method  */

}/*  end of NonNeighboursOnly class  */
