package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;


/** ------------------------------------------------------------------------------------------------------------------
 * start of ProbeMatcher class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class ProbeMatcher {
	
	
	
	
	/*  global parameters  */
	private static final String inputPath = "ProbeMatcher/Input/";
	private static final String outputPath = "ProbeMatcher/Output/";
	private static ArrayList<String[]> matched_file_list;
	/*  end of global parameters  */
	
	

	
	/*  start of getFilenames method  */
	/**
	 * @param filename_file_list the file containing the list of files
	 */
	private static void getFilenames(String filename_file_list){
		if(filename_file_list == null){
			System.err.println("Error: the filename list is null.\nSystem.exit(0)");
			System.exit(0);
		}
		
		matched_file_list = new ArrayList<String[]>();

		String line = null;
		int lineCounter = -1;			//  there is one header line
		try{
			BufferedReader br = new BufferedReader(new FileReader(inputPath + filename_file_list));
			while((line = br.readLine()) != null){
				if(lineCounter >= 0){
					String[] values = line.split("\t");
					if(values.length != 2){
						System.err.println("Error: there are not exactly two (2) entries in each line of the file list.\nSystem.exit(0)");
						System.exit(0);
					}
					String[] currentMatchedPair = new String[2];
					currentMatchedPair[0] = values[0];
					currentMatchedPair[1] = values[1];
					
					matched_file_list.add(currentMatchedPair);
				}
				else{
					//  skip and the header line
//					System.out.println(line);
				}
				lineCounter++;
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
		}
		matched_file_list.trimToSize();
		System.out.println("Loaded input file: " + inputPath + filename_file_list);
	}/*  end of getFilenames method  */
	
	

	
	/*  start of displayFilenames method  */
	/**
	 * @return void
	 */
	private static void displayFilenames(){
		if(matched_file_list == null){
			System.err.println("Error: the file list is null and cannot be displayed.\nSystem.exit(0)");
			System.exit(0);	
		}
		
		System.out.println("Target\tSuperset to be thinned");
		for(String[] currentFilename: matched_file_list){
			System.out.println(currentFilename[0] + "\t" + currentFilename[1]);
		}
	}/*  end of displayFilenames method  */
	
	
	
	
	/*  start of matchProbesAndOutputToFile method  */
	/**
	 * @return void
	 */
	private static void matchProbesAndOutputToFile(){
		if(matched_file_list == null){
			System.err.println("Error: the file list is null and the probes cannot be matched and thinned.\nSystem.exit(0)");
			System.exit(0);
		}
		
		
		for(String[] currentMatchedPair: matched_file_list){
			
			String targetFilename = currentMatchedPair[0];
			String supersetFilename = currentMatchedPair[1];
			
//			String outputFilename = "matched_" + supersetFilename;
			String outputFilename = supersetFilename;
			
			ArrayList<String> probesInTarget = new ArrayList<String>();
			
			String line = null;
			String header = "";
			int lineCounter = -1;			//  there is one header line
			try{
				BufferedReader br = new BufferedReader(new FileReader(inputPath + targetFilename));
				while((line = br.readLine()) != null){
					if(lineCounter >= 0){
						String[] values = line.split("\t");
						probesInTarget.add(values[0]);
					}
					else{
						header = line;
					}
					lineCounter++;
				}
				br.close();
			}catch (IOException e) {
				e.printStackTrace();
			}
			System.out.println("Loaded input file: " + inputPath + targetFilename);
			probesInTarget.trimToSize();
			
			
			line = null;
			lineCounter = -1;			//  there is one header line
			try{
				BufferedReader br = new BufferedReader(new FileReader(inputPath + supersetFilename));
				BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath + outputFilename));
				while((line = br.readLine()) != null){
					if(lineCounter >= 0){
						String[] values = line.split("\t");
						if(probesInTarget.contains(values[0])){
							bw.write(line);
							bw.newLine();
							bw.flush();
						}
					}
					else{
						if(header.compareTo(line) == 0){
							bw.write(line);
							bw.newLine();
							bw.flush();
						}
						else{
							System.err.println("Error: the header lines in the two input files do not match.\nSystem.exit(0)");
							System.exit(0);
						}
					}
					lineCounter++;
				}
				br.close();
				bw.flush();
				bw.close();
			}catch (IOException e) {
				e.printStackTrace();
			}
			System.out.println("Loaded input file: " + inputPath + supersetFilename);
			System.out.println("Output file: " + outputPath + outputFilename);
		}
	}/*  end of matchProbesAndOutputToFile method  */
	
	
	
	
	/*  start of main method  */
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		
		

		/******************************************************
		 * 													  *
		 * 				SET THE PARAMETERS HERE				  *
		 * 													  *
		 ******************************************************/
		
		
		String filename_file_list = "file_list.txt";
		
		
		/******************************************************
		 * 													  *
		 * 				END OF SETTING THE PARAMETERS		  *
		 * 													  * 
		 ******************************************************/
		
		
		

		getFilenames(filename_file_list);
//		displayFilenames();
		
		
		matchProbesAndOutputToFile();
		
		
		System.out.println("Done!");
	}/*  end of main method  */

}/*  end of ProbeMatcher class  */
