package utilities;


/**
 * @author Mathieu Rousseau
 * @email mathieu.rousseau3@mail.mcgill.ca
 */


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;


/** ------------------------------------------------------------------------------------------------------------------
 * start of RemoveExcludedFragmentPairs class
 * ------------------------------------------------------------------------------------------------------------------  **/
public class RemoveExcludedFragmentPairs {

	
	
	
	/*  global parameters  */
	private static final String inputPath = "RemoveExcludedFragmentPairs/Input/";
	private static final String outputPath = "RemoveExcludedFragmentPairs/Output/";
	private static ArrayList<String> exclusion_list;
	/*  end of global parameters  */
	
	

	
	/*  start of loadExclusionListFromFile method  */
	/**
	 * @param filename the filename of the file containing the exclusion list
	 */
	private static void loadExclusionListFromFile(String filename){
		//  make sure that the input filename is specified 
		if(filename == null){
			System.err.println("Error: the exclusion list filename is not specified.\nSystem.exit(0)");
			System.exit(0);
		}
		
		exclusion_list = new ArrayList<String>();
		
		String line = null;
		
		try{
			BufferedReader br = new BufferedReader(new FileReader(inputPath + filename));
			while((line = br.readLine()) != null){
				exclusion_list.add(line);
			}
			br.close();
		}catch (IOException e) {
			e.printStackTrace();
			System.err.println("Error: cannot find file with name \"" + filename + "\"\nSystem.exit(0)");
			System.exit(0);
		}
		exclusion_list.trimToSize();
		System.out.println("\tOpened and read file: " + filename);
	}/*  end of loadExclusionListFromFile method  */
	
	
	
	
	/*  start of cleanInputFilesAndOutputToFile method  */
	/**
	 * @param filenames the list of filenames for the input files to clean
	 */
	private static void cleanInputFilesAndOutputToFile(ArrayList<String> filenames){
		//  make sure that the input filenames are specified 
		if((filenames == null) || (filenames.size() < 1)){
			System.err.println("Error: the filenames for the files to clean are not specified.\nSystem.exit(0)");
			System.exit(0);
		}
		
		for(String filename_current_input_file: filenames){
			String filename_current_output_file = filename_current_input_file;
			String line = null;
			int numberOfEntries_input_file = 0;
			int numberOfEntries_output_file = 0;
			int numberOfEntries_excluded = 0;
			try{
				BufferedReader br = new BufferedReader(new FileReader(inputPath + filename_current_input_file));
				BufferedWriter bw = new BufferedWriter(new FileWriter(outputPath + filename_current_output_file));
				while((line = br.readLine()) != null){
					numberOfEntries_input_file++;
					String[] values = line.split("\t");
					if(values.length == 10){
						//  parse the values once that will be used to save time
						int firstIndex = Integer.parseInt(values[0]);
						int secondIndex = Integer.parseInt(values[1]);
						
						String fragmentPairToTest = firstIndex + "-" + secondIndex;
						String fragmentPairToTestReverse = secondIndex + "-" + firstIndex;

						if((!exclusion_list.contains(fragmentPairToTest)) && (!exclusion_list.contains(fragmentPairToTestReverse))){
							//  put the entry in the cleaned output file
							bw.write(line);
							bw.newLine();
							bw.flush();
							numberOfEntries_output_file++;
						}
						else{
							//  remove the entry from the cleaned output file
							numberOfEntries_excluded++;
						}
					}
					else{
						System.err.println("Error: there are not ten (10) entries in each row of the file \"" + filename_current_input_file + "\"\nSystem.exit(0)");
						System.exit(0);
					}
				}
				br.close();
				bw.close();
			}catch (IOException e) {
				e.printStackTrace();
				System.err.println("Error: cannot find file with name \"" + filename_current_input_file + "\"\nSystem.exit(0)");
				System.err.println("Error: cannot output file with name \"" + filename_current_output_file + "\"\nSystem.exit(0)");
				System.exit(0);
			}			
			System.out.println("\tOpened and read file: " + filename_current_input_file);		
			System.out.println("\tOutput file: " + filename_current_output_file);
			System.out.println("Number of entries in input file: " + filename_current_input_file + " =\t" + numberOfEntries_input_file);
			System.out.println("Number of entries in output file: " + filename_current_output_file + " =\t" + numberOfEntries_output_file);
			System.out.println("Number of entries excluded: =\t" + numberOfEntries_excluded);
			System.out.println("\t[" + numberOfEntries_output_file + " + " + numberOfEntries_excluded + " ?= " + numberOfEntries_input_file + "]");
		}
	}/*  end of cleanInputFilesAndOutputToFile method  */
	
	
	
	
	/*  start of displayExclusionList method  */
	/**
	 * @return void
	 */
	private static void displayExclusionList(){
		if(exclusion_list == null){
			System.err.println("Error: trying to display the exclusion list that is null.\nSystem.exit(0)");
			System.exit(0);
		}
		for(String currentEntry: exclusion_list){
			System.out.println(currentEntry);
		}
	}/*  end of displayExclusionList method  */
	
	
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		

		ArrayList<String> filenames_input_to_clean = new ArrayList<String>();
		String filename_exclusion_list = "";
		
		if(args.length > 0){
			filename_exclusion_list = args[0];
			
			for(int i=1; i<args.length; i++){
				filenames_input_to_clean.add(args[i]);
			}
		}
		else{
			
			
//			filenames_input_to_clean.add("DMSO_GD.txt");
//			filenames_input_to_clean.add("PMA_GD.txt");
//			filename_exclusion_list = "exclusion_list_GeneDesert.txt";
			
//			filenames_input_to_clean.add("DMSO_HoxA.txt");
//			filenames_input_to_clean.add("PMA_HoxA.txt");
//			filename_exclusion_list = "exclusion_list_HoxA.txt";
		
		
//			filenames_input_to_clean.add("BAC01.txt");
//			filenames_input_to_clean.add("BAC02.txt");
//			filenames_input_to_clean.add("BAC03.txt");
//			filenames_input_to_clean.add("BAC04.txt");
//			filenames_input_to_clean.add("BAC05.txt");
//			filenames_input_to_clean.add("BAC06.txt");
//			filenames_input_to_clean.add("BAC07.txt");
//			filenames_input_to_clean.add("BAC08.txt");
//			filename_exclusion_list = "exclusion_list_HoxA.txt";
//			filename_exclusion_list = "exclusion_list_GeneDesert.txt";
		
		
//			filenames_input_to_clean.add("0Hr_HOXAHOXA.txt");
//			filenames_input_to_clean.add("0Hr_2_HOXAHOXA.txt");
//			filenames_input_to_clean.add("4Hr_HOXAHOXA.txt");
//			filenames_input_to_clean.add("4Hr_2_HOXAHOXA.txt");
//			filenames_input_to_clean.add("12Hr_HOXAHOXA.txt");
//			filenames_input_to_clean.add("48Hr_HOXAHOXA.txt");
//			filenames_input_to_clean.add("96Hr_HOXAHOXA.txt");
//			filename_exclusion_list = "exclusion_list_HoxA.txt";
		
		
//			filenames_input_to_clean.add("0Hr_GDGD.txt");
//			filenames_input_to_clean.add("0Hr_2_GDGD.txt");
//			filenames_input_to_clean.add("4Hr_GDGD.txt");
//			filenames_input_to_clean.add("4Hr_2_GDGD.txt");
//			filenames_input_to_clean.add("12Hr_GDGD.txt");
//			filenames_input_to_clean.add("48Hr_GDGD.txt");
//			filenames_input_to_clean.add("96Hr_GDGD.txt");
//			filename_exclusion_list = "exclusion_list_GeneDesert.txt";
		
		
		
			
//			filenames_input_to_clean.add("BM-A-1_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-A-2_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-A-3_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-AF9-1_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-AF9-2_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-H-2_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-H-3_HOXAHOXA.txt");
//			filenames_input_to_clean.add("BM-H-4_HOXAHOXA.txt");
//			filenames_input_to_clean.add("Nalm-6-1_HOXAHOXA.txt");
//			filenames_input_to_clean.add("Nalm-6-2_HOXAHOXA.txt");
//			filenames_input_to_clean.add("NB-4-1_HOXAHOXA.txt");
//			
//			filename_exclusion_list = "exclusion_list_HoxA.txt";
		
		

//			filenames_input_to_clean.add("BM-A-1_GDGD.txt");
//			filenames_input_to_clean.add("BM-A-2_GDGD.txt");
//			filenames_input_to_clean.add("BM-A-3_GDGD.txt");
//			filenames_input_to_clean.add("BM-AF9-1_GDGD.txt");
//			filenames_input_to_clean.add("BM-AF9-2_GDGD.txt");
//			filenames_input_to_clean.add("BM-H-2_GDGD.txt");
//			filenames_input_to_clean.add("BM-H-3_GDGD.txt");
//			filenames_input_to_clean.add("BM-H-4_GDGD.txt");
//			filenames_input_to_clean.add("Nalm-6-1_GDGD.txt");
//			filenames_input_to_clean.add("Nalm-6-2_GDGD.txt");
//			filenames_input_to_clean.add("NB-4-1_GDGD.txt");
//			
//			filename_exclusion_list = "exclusion_list_GeneDesert.txt";
		
		
		
//			filenames_input_to_clean.add("BAC01_Mathieu.txt");
//			filenames_input_to_clean.add("BAC02_Mathieu.txt");
//			filenames_input_to_clean.add("BAC03_Mathieu.txt");
//		
//			filenames_input_to_clean.add("BAC03_Jennifer.txt");
//		
//			filenames_input_to_clean.add("BAC02_Maria.txt");
//			filenames_input_to_clean.add("BAC03_Maria.txt");
//			filenames_input_to_clean.add("BAC04_Maria.txt");
//			filenames_input_to_clean.add("BAC05_Maria.txt");
//			filenames_input_to_clean.add("BAC06_Maria.txt");
//		
//			filename_exclusion_list = "exclusion_list_HoxA.txt";
//			filename_exclusion_list = "exclusion_list_GeneDesert.txt";
		


			
//			filenames_input_to_clean.add("cleaned_BAC01_Mathieu_HOXAHOXA.txt");
//			filenames_input_to_clean.add("cleaned_BAC02_Mathieu_HOXAHOXA.txt");
//			filenames_input_to_clean.add("cleaned_BAC03_Mathieu_HOXAHOXA.txt");
//
//			filenames_input_to_clean.add("cleaned_BAC03_Jennifer_HOXAHOXA.txt");
//
//			filenames_input_to_clean.add("cleaned_BAC02_Maria_HOXAHOXA.txt");
//			filenames_input_to_clean.add("cleaned_BAC03_Maria_HOXAHOXA.txt");
//			filenames_input_to_clean.add("cleaned_BAC04_Maria_HOXAHOXA.txt");
//			filenames_input_to_clean.add("cleaned_BAC05_Maria_HOXAHOXA.txt");
//			filenames_input_to_clean.add("cleaned_BAC06_Maria_HOXAHOXA.txt");
//
//			filename_exclusion_list = "exclusion_list_HoxA.txt";
			
			
//			filenames_input_to_clean.add("cleaned_BAC01_Mathieu_GDGD.txt");
//			filenames_input_to_clean.add("cleaned_BAC02_Mathieu_GDGD.txt");
//			filenames_input_to_clean.add("cleaned_BAC03_Mathieu_GDGD.txt");
//
//			filenames_input_to_clean.add("cleaned_BAC03_Jennifer_GDGD.txt");
//
//			filenames_input_to_clean.add("0h_biological2_GDGD.txt");
//			filenames_input_to_clean.add("0h_technical_GDGD.txt");
//			filenames_input_to_clean.add("4h_biological2_GDGD.txt");
//			filenames_input_to_clean.add("4h_technical_3_GDGD.txt");
//			filenames_input_to_clean.add("12h_biological2_GDGD.txt");
//			filenames_input_to_clean.add("12h_technical_GDGD.txt");
//			filenames_input_to_clean.add("48h_biological2_GDGD.txt");
//			filenames_input_to_clean.add("48h_technical_GDGD.txt");
//			filenames_input_to_clean.add("96h_biological2_GDGD.txt");
//			filenames_input_to_clean.add("96h_technical_GDGD.txt");
//			filename_exclusion_list = "exclusion_list_GeneDesert.txt";
			
			filenames_input_to_clean.add("0h_biological2_HOXAHOXA.txt");			
			filenames_input_to_clean.add("0h_technical_HOXAHOXA.txt");			
			filenames_input_to_clean.add("4h_biological2_HOXAHOXA.txt");			
			filenames_input_to_clean.add("4h_technical_3_HOXAHOXA.txt");			
			filenames_input_to_clean.add("12h_biological2_HOXAHOXA.txt");			
			filenames_input_to_clean.add("12h_technical_HOXAHOXA.txt");			
			filenames_input_to_clean.add("48h_biological2_HOXAHOXA.txt");			
			filenames_input_to_clean.add("48h_technical_HOXAHOXA.txt");
			filenames_input_to_clean.add("96h_biological2_HOXAHOXA.txt");			
			filenames_input_to_clean.add("96h_technical_HOXAHOXA.txt");
			filename_exclusion_list = "exclusion_list_HoxA.txt";
		}
		
		
		loadExclusionListFromFile(filename_exclusion_list);
//		displayExclusionList();
		
		
		
		cleanInputFilesAndOutputToFile(filenames_input_to_clean);
		
		
		
		System.out.println("Done!");
	}

}/*  end of RemoveExcludedFragmentPairs class  */